/*  zoomview.cpp - the zoom component for the AtheOS Zoom v0.1
 *  Copyright (C) 2001 Chir
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details. 
 *
 *  You should have received a copy of the GNU Library General Public 
 *  License along with this library; if not, write to the Free 
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include "zoomview.h"


using namespace os;


// ---------------------------------------------------------------------------

ZoomView::ZoomView( Rect cFrame ) : LayoutView( cFrame, "my_view" )
{
    m_bIsFirst = true;
    m_H = 3;
    m_pBufferRed = NULL;  m_pBufferGreen = NULL;  m_pBufferBlue = NULL;
    SetEraseColor( 0, 0, 0 );         // Black background.
}

ZoomView::~ZoomView()
{
    if( m_pBufferRed != NULL )
    {
        for( int i=0; i<2*m_RX+1; ++i )
        {
            delete m_pBufferRed[i];  delete m_pBufferGreen[i];  delete m_pBufferBlue[i];
        }
        delete m_pBufferRed;  delete m_pBufferGreen;  delete m_pBufferBlue;
    }
}

void ZoomView::Paint( const Rect& cUpdateRect )
{
    if( m_bIsFirst )
    {
        // Defered initialisation.
    m_bIsFirst = false;
        GetWindow()->AddTimer( this, 0, 1500000, false );
        resizeZoom();
    }
    FillRect( cUpdateRect );         // Clear area.
    drawPixels();
}

Point ZoomView::GetPreferredSize( bool bLargest )
{
    return Point(100,100);
}

void ZoomView::FrameSized( const Point& cDelta )
{
   resizeZoom();
   Invalidate(true);       // Refresh.
}

void ZoomView::TimerTick( int nID )
{
   fetchPixels();
}

/*
 * Changes the zoom level.
 */
void ZoomView::setMagnification( int nSize )
{
   m_H = nSize;
   resizeZoom();
   Invalidate(true);       // Refresh.
}

/**
 * Fills the pixel buffer, and displays it.
 */
void ZoomView::fetchPixels()
{
    uint32 nButtons;
    Point cPosition1 , cPosition2;
    Desktop cMyScreen;
    screen_mode sMode = cMyScreen.GetScreenMode();
    unsigned char* pBuffer = (unsigned char*)(cMyScreen.GetFrameBuffer());
    int nWidth = sMode.m_nWidth , nHeight = sMode.m_nHeight;
    int nDepth = sMode.m_nBytesPerLine / nWidth;

    if( m_bIsFirst )  return;                  // Not initialised yet.

    GetMouse( &cPosition1, &nButtons );
   
    if( cPosition1.x >= 0  &&  cPosition1.x < Width()  &&  cPosition1.y >= 0  &&  cPosition1.y < Height() )  return;
   
    cPosition2 = ConvertToScreen( cPosition1 );

    int nMouseX = (int) cPosition2.x , nMouseY = (int) cPosition2.y;
   
    // Respect screen limits.
    if( nMouseX - m_RX <= 0 )  nMouseX = m_RX;
    if( nMouseX + m_RX >= nWidth )  nMouseX = nWidth - m_RX;
    if( nMouseY - m_RY <= 0 )  nMouseY = m_RY;
    if( nMouseY + m_RY >= nHeight )  nMouseY = nHeight - m_RY;

nDepth = 2;     // !!! Force this color depth !!!
    for( int i=-m_RX; i<=m_RX; ++i )
    {
        for( int j=-m_RY; j<=m_RY; ++j )
        {
            unsigned char* p = &(pBuffer[ (nMouseY + j) * nDepth*nWidth  +  (nMouseX + i) * nDepth ]);
            int nValue = ((*p)<<0) | ((*(++p)) << 8 );
            int nBlue = (nValue & 0x1F) << 3;
            int nGreen = ((nValue >> 5) & 0x3F) << 2;
            int nRed = ((nValue >> 11) & 0x1F) << 3;

            m_pBufferRed[i+m_RX][j+m_RY] = nRed;
            m_pBufferGreen[i+m_RX][j+m_RY] = nGreen;
            m_pBufferBlue[i+m_RX][j+m_RY] = nBlue;
        }
    }
    drawPixels();
}

/**
 * Displays the contents of the pixel buffer.
 */
void ZoomView::drawPixels()
{ 
    for( int i=-m_RX; i<=m_RX; ++i )
    {
        for( int j=-m_RY; j<=m_RY; ++j )
        {
            int nRed = m_pBufferRed[ i + m_RX ][ j + m_RY ];
            int nGreen = m_pBufferGreen[ i + m_RX ][ j + m_RY ];
            int nBlue = m_pBufferBlue[ i + m_RX ][ j + m_RY ];
       
            FillRect( Rect(m_vX0+m_H*i,m_vY0+m_H*j,m_vX0+m_H*(i+1)-1,m_vY0+m_H*(j+1)-1) , Color32_s(nRed,nGreen,nBlue) );
        }
    }
    Flush();        // Ensure everything gets drawn right now.
}

/*
 * Creates a new (empty) pixel buffer, and fills it.
 */
void ZoomView::resizeZoom()
{
    if( m_pBufferRed != NULL )
    {
        for( int i=0; i<2*m_RX+1; ++i )
        {
            delete m_pBufferRed[i];  delete m_pBufferGreen[i];  delete m_pBufferBlue[i];
        }
        delete m_pBufferRed;  delete m_pBufferGreen;  delete m_pBufferBlue;
    }
    m_vX0 = Width()/2 ;  m_vY0 = Height()/2 ;             // Center of rectangle.
    m_RX = ((int) Width() / 2) / m_H ;
    m_RY = ((int) Height() / 2) / m_H ;
    m_pBufferRed = new (unsigned char*)[2*m_RX+1];
    m_pBufferGreen = new (unsigned char*)[2*m_RX+1];
    m_pBufferBlue = new (unsigned char*)[2*m_RX+1];
    for( int i=0; i<2*m_RX+1; ++i )
    {
       m_pBufferRed[i] = new unsigned char[2*m_RY+1];
       m_pBufferGreen[i] = new unsigned char[2*m_RY+1];
       m_pBufferBlue[i] = new unsigned char[2*m_RY+1];
       for( int j=0; j<2*m_RY+1; ++j )
       {
      m_pBufferRed[i][j] = 0;  m_pBufferGreen[i][j] = 0;  m_pBufferBlue[i][j] = 0;
       }
    }
    fetchPixels();              // Fill the buffer.
}

// ---------------------------------------------------------------------------
